package net.sf.tomp.pp;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.HashMap;
import java.util.Map;

import net.sf.tomp.xml.sax.ParametrizedXMLReader;

import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;

/**
 * DOCUMENT ME!
 * 
 * @author $author$
 * @version $Revision$
 */
public abstract class AbstractXMLReader implements ParametrizedXMLReader {
    /** DOCUMENT ME! */
    protected Map featureMap = new HashMap();

    /** DOCUMENT ME! */
    protected Map propertyMap = new HashMap();

    /** DOCUMENT ME! */
    protected EntityResolver entityResolver;

    /** DOCUMENT ME! */
    protected ContentHandler contentHandler;

    /** DOCUMENT ME! */
    protected ErrorHandler errorHandler;

    /** DOCUMENT ME! */
    protected DTDHandler dtdHandler;

    /**
     * DOCUMENT ME!
     * 
     * @param k DOCUMENT ME!
     * @param v DOCUMENT ME!
     */
    public abstract void setParameter(String k, Object v);

    /**
     * DOCUMENT ME!
     * 
     * @param name DOCUMENT ME!
     * @return DOCUMENT ME!
     * @throws SAXNotRecognizedException DOCUMENT ME!
     * @throws SAXNotSupportedException DOCUMENT ME!
     */
    public boolean getFeature(String name) throws SAXNotRecognizedException,
            SAXNotSupportedException {
        if (featureMap.containsKey(name)) {
            return ((Boolean) featureMap.get(name)).booleanValue();
        }

        return false;
    }

    /**
     * DOCUMENT ME!
     * 
     * @param name DOCUMENT ME!
     * @param value DOCUMENT ME!
     * @throws SAXNotRecognizedException DOCUMENT ME!
     * @throws SAXNotSupportedException DOCUMENT ME!
     */
    public void setFeature(String name, boolean value)
            throws SAXNotRecognizedException, SAXNotSupportedException {
        featureMap.put(name, new Boolean(value));
    }

    /**
     * DOCUMENT ME!
     * 
     * @param name DOCUMENT ME!
     * @return DOCUMENT ME!
     * @throws SAXNotRecognizedException DOCUMENT ME!
     * @throws SAXNotSupportedException DOCUMENT ME!
     */
    public Object getProperty(String name) throws SAXNotRecognizedException,
            SAXNotSupportedException {
        return propertyMap.get(name);
    }

    /**
     * DOCUMENT ME!
     * 
     * @param name DOCUMENT ME!
     * @param value DOCUMENT ME!
     * @throws SAXNotRecognizedException DOCUMENT ME!
     * @throws SAXNotSupportedException DOCUMENT ME!
     */
    public void setProperty(String name, Object value)
            throws SAXNotRecognizedException, SAXNotSupportedException {
        propertyMap.put(name, value);
    }

    /**
     * DOCUMENT ME!
     * 
     * @param resolver DOCUMENT ME!
     */
    public void setEntityResolver(EntityResolver resolver) {
        entityResolver = resolver;
    }

    /**
     * DOCUMENT ME!
     * 
     * @return DOCUMENT ME!
     */
    public EntityResolver getEntityResolver() {
        return entityResolver;
    }

    /**
     * DOCUMENT ME!
     * 
     * @param handler DOCUMENT ME!
     */
    public void setContentHandler(ContentHandler handler) {
        contentHandler = handler;
    }

    /**
     * DOCUMENT ME!
     * 
     * @return DOCUMENT ME!
     */
    public ContentHandler getContentHandler() {
        return contentHandler;
    }

    /**
     * DOCUMENT ME!
     * 
     * @param handler DOCUMENT ME!
     */
    public void setErrorHandler(ErrorHandler handler) {
        errorHandler = handler;
    }

    /**
     * DOCUMENT ME!
     * 
     * @return DOCUMENT ME!
     */
    public ErrorHandler getErrorHandler() {
        return errorHandler;
    }

    /**
     * DOCUMENT ME!
     * 
     * @param handler DOCUMENT ME!
     */
    public void setDTDHandler(DTDHandler handler) {
        dtdHandler = handler;
    }

    /**
     * DOCUMENT ME!
     * 
     * @return DOCUMENT ME!
     */
    public DTDHandler getDTDHandler() {
        return dtdHandler;
    }

    /**
     * DOCUMENT ME!
     * 
     * @param systemId DOCUMENT ME!
     * @throws IOException DOCUMENT ME!
     * @throws SAXException DOCUMENT ME!
     */
    public void parse(String systemId) throws IOException, SAXException {
        parse(new InputSource(systemId));
    }

    /**
     * DOCUMENT ME!
     * 
     * @param source DOCUMENT ME!
     * @throws IOException DOCUMENT ME!
     * @throws SAXException DOCUMENT ME!
     */
    public abstract void parse(InputSource source) throws IOException,
            SAXException;

    /**
     * DOCUMENT ME!
     * 
     * @param source DOCUMENT ME!
     * @return DOCUMENT ME!
     * @throws IOException DOCUMENT ME!
     */
    protected File getFile(InputSource source) throws IOException {
        String systemid = source.getSystemId();

        if (systemid != null) {
            return new File(systemid);
        }

        return null;
    }

    /**
     * DOCUMENT ME!
     * 
     * @param source DOCUMENT ME!
     * @return DOCUMENT ME!
     * @throws IOException DOCUMENT ME!
     */
    protected InputStream getInputStream(InputSource source) throws IOException {
        InputStream input = source.getByteStream();

        if (input != null) {
            return input;
        }

        String systemid = source.getSystemId();

        if (systemid != null) {
            return new FileInputStream(systemid);
        }

        return null;
    }

    /**
     * DOCUMENT ME!
     * 
     * @param source DOCUMENT ME!
     * @return DOCUMENT ME!
     * @throws IOException DOCUMENT ME!
     */
    protected BufferedReader getBufferedReader(InputSource source)
            throws IOException {
        Reader reader = source.getCharacterStream();

        if (reader != null) {
            return new BufferedReader(reader);
        }

        InputStream input = getInputStream(source);

        if (input != null) {
            reader = new InputStreamReader(input);

            return new BufferedReader(reader);
        }

        return null;
    }
}
/*
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License. The
 * Original Code is: all this file. The Initial Developer of the Original Code
 * is: Tomas Pitner, Masaryk University in Brno, Czech Republic. Contributor(s):
 */
